@@ -1,13 +1,14 @@
-use std::str::FromStr;
-
-use crate::Entity;
+use crate::{ActiveEnum, Entity};
+use heck::CamelCase;
 use proc_macro2::TokenStream;
-use quote::quote;
+use quote::{format_ident, quote};
+use std::{collections::HashMap, str::FromStr};
 use syn::{punctuated::Punctuated, token::Comma};
 
 #[derive(Clone, Debug)]
 pub struct EntityWriter {
     pub(crate) entities: Vec<Entity>,
+    pub(crate) enums: HashMap<String, ActiveEnum>,
 }
 
 pub struct WriterOutput {
@@ -83,6 +84,9 @@ impl EntityWriter {
         files.extend(self.write_entities(expanded_format, with_serde));
         files.push(self.write_mod());
         files.push(self.write_prelude());
+        if !self.enums.is_empty() {
+            files.push(self.write_sea_orm_active_enums());
+        }
         WriterOutput { files }
     }
 
@@ -109,11 +113,7 @@ impl EntityWriter {
     pub fn write_mod(&self) -> OutputFile {
         let mut lines = Vec::new();
         Self::write_doc_comment(&mut lines);
-        let code_blocks: Vec<TokenStream> = self
-            .entities
-            .iter()
-            .map(|entity| Self::gen_mod(entity))
-            .collect();
+        let code_blocks: Vec<TokenStream> = self.entities.iter().map(Self::gen_mod).collect();
         Self::write(
             &mut lines,
             vec![quote! {
@@ -122,6 +122,14 @@ impl EntityWriter {
         );
         lines.push("".to_owned());
         Self::write(&mut lines, code_blocks);
+        if !self.enums.is_empty() {
+            Self::write(
+                &mut lines,
+                vec![quote! {
+                    pub mod sea_orm_active_enums;
+                }],
+            );
+        }
         OutputFile {
             name: "mod.rs".to_owned(),
             content: lines.join("\n"),
@@ -131,11 +139,7 @@ impl EntityWriter {
     pub fn write_prelude(&self) -> OutputFile {
         let mut lines = Vec::new();
         Self::write_doc_comment(&mut lines);
-        let code_blocks = self
-            .entities
-            .iter()
-            .map(|entity| Self::gen_prelude_use(entity))
-            .collect();
+        let code_blocks = self.entities.iter().map(Self::gen_prelude_use).collect();
         Self::write(&mut lines, code_blocks);
         OutputFile {
             name: "prelude.rs".to_owned(),
@@ -143,6 +147,28 @@ impl EntityWriter {
         }
     }
 
+    pub fn write_sea_orm_active_enums(&self) -> OutputFile {
+        let mut lines = Vec::new();
+        Self::write_doc_comment(&mut lines);
+        Self::write(
+            &mut lines,
+            vec![quote! {
+                use sea_orm::entity::prelude::*;
+            }],
+        );
+        lines.push("".to_owned());
+        let code_blocks = self
+            .enums
+            .iter()
+            .map(|(_, active_enum)| active_enum.impl_active_enum())
+            .collect();
+        Self::write(&mut lines, code_blocks);
+        OutputFile {
+            name: "sea_orm_active_enums.rs".to_owned(),
+            content: lines.join("\n"),
+        }
+    }
+
     pub fn write(lines: &mut Vec<String>, code_blocks: Vec<TokenStream>) {
         lines.extend(
             code_blocks
@@ -163,8 +189,10 @@ impl EntityWriter {
     }
 
     pub fn gen_expanded_code_blocks(entity: &Entity, with_serde: &WithSerde) -> Vec<TokenStream> {
+        let mut imports = Self::gen_import(with_serde);
+        imports.extend(Self::gen_import_active_enum(entity));
         let mut code_blocks = vec![
-            Self::gen_import(with_serde),
+            imports,
             Self::gen_entity_struct(),
             Self::gen_impl_entity_name(entity),
             Self::gen_model_struct(entity, with_serde),
@@ -182,10 +210,9 @@ impl EntityWriter {
     }
 
     pub fn gen_compact_code_blocks(entity: &Entity, with_serde: &WithSerde) -> Vec<TokenStream> {
-        let mut code_blocks = vec![
-            Self::gen_import(with_serde),
-            Self::gen_compact_model_struct(entity, with_serde),
-        ];
+        let mut imports = Self::gen_import(with_serde);
+        imports.extend(Self::gen_import_active_enum(entity));
+        let mut code_blocks = vec![imports, Self::gen_compact_model_struct(entity, with_serde)];
         let relation_defs = if entity.get_relation_enum_name().is_empty() {
             vec![
                 Self::gen_relation_enum(entity),
@@ -249,6 +276,21 @@ impl EntityWriter {
         }
     }
 
+    pub fn gen_import_active_enum(entity: &Entity) -> TokenStream {
+        entity
+            .columns
+            .iter()
+            .fold(TokenStream::new(), |mut ts, col| {
+                if let sea_query::ColumnType::Enum(enum_name, _) = &col.col_type {
+                    let enum_name = format_ident!("{}", enum_name.to_camel_case());
+                    ts.extend(vec![quote! {
+                        use super::sea_orm_active_enums::#enum_name;
+                    }]);
+                }
+                ts
+            })
+    }
+
     pub fn gen_model_struct(entity: &Entity, with_serde: &WithSerde) -> TokenStream {
         let column_names_snake_case = entity.get_column_names_snake_case();
         let column_rs_types = entity.get_column_rs_types();
@@ -264,11 +306,22 @@ impl EntityWriter {
     }
 
     pub fn gen_column_enum(entity: &Entity) -> TokenStream {
-        let column_names_camel_case = entity.get_column_names_camel_case();
+        let column_variants = entity.columns.iter().map(|col| {
+            let variant = col.get_name_camel_case();
+            let mut variant = quote! { #variant };
+            if !col.is_snake_case_name() {
+                let column_name = &col.name;
+                variant = quote! {
+                    #[sea_orm(column_name = #column_name)]
+                    #variant
+                };
+            }
+            variant
+        });
         quote! {
             #[derive(Copy, Clone, Debug, EnumIter, DeriveColumn)]
             pub enum Column {
-                #(#column_names_camel_case,)*
+                #(#column_variants,)*
             }
         }
     }
@@ -432,6 +485,10 @@ impl EntityWriter {
             .iter()
             .map(|col| {
                 let mut attrs: Punctuated<_, Comma> = Punctuated::new();
+                if !col.is_snake_case_name() {
+                    let column_name = &col.name;
+                    attrs.push(quote! { column_name = #column_name });
+                }
                 if primary_keys.contains(&col.name) {
                     attrs.push(quote! { primary_key });
                     if !col.auto_increment {
@@ -682,14 +739,14 @@ mod tests {
                         unique: false,
                     },
                     Column {
-                        name: "name".to_owned(),
+                        name: "_name_".to_owned(),
                         col_type: ColumnType::String(Some(255)),
                         auto_increment: false,
                         not_null: true,
                         unique: false,
                     },
                     Column {
-                        name: "fruit_id".to_owned(),
+                        name: "fruitId".to_owned(),
                         col_type: ColumnType::Integer(Some(11)),
                         auto_increment: false,
                         not_null: false,
@@ -698,7 +755,7 @@ mod tests {
                 ],
                 relations: vec![Relation {
                     ref_table: "fruit".to_owned(),
-                    columns: vec!["fruit_id".to_owned()],
+                    columns: vec!["fruitId".to_owned()],
                     ref_columns: vec!["id".to_owned()],
                     rel_type: RelationType::BelongsTo,
                     on_delete: None,
